/**
 * A partial definition of a {@link GalleryEntry}
 * which defines the properties of an entry
 * that can be passed down from a top-level entry down to its variants
 */
export type GalleryEntryInheritedProperties = {
  /**
   * [SUPPORTS MARKDOWN] a place for the siteowner to describe the entry
   */
  description?: string
  /**
   * a key-value pair set of general-purpose fields to additionally describe the entry
   * @example entry.fields = {
   *   'date': '1960/01/01',
   *   'rating': 'general',
   * }
   */
  fields?: Record<string, string>
  /**
   * array of tag-ids that apply to the art;
   * this is used to allow a visitor to filter out entries of a specific tag
   * @see {@link GalleryList.tags}
   */
  tags: string[]
  /**
   * the title of the entry
   */
  title: string | null | undefined
  /**
   * any content warnings that apply to the entry,
   * which will be used to apply the `.warning` classname
   * to the DOM of a gallery tile
   * and will be displayed with the tile
   */
  warning?: string
}

/**
 * Defines an entry in a gallery that can be displayed in a tiled manner
 * and can be clicked by a visitor to display its variants or the entry itself if there are none
 */
export type GalleryEntry = GalleryEntryInheritedProperties & {
  /**
   * the url to the thumbnail to show for the entry in the gallery tile
   */
  thumbnailUrl?: string
  /**
   * the url to the entry itself
   */
  url?: string
  /**
   * optional variants for the entry;
   * this is a recursive definition of {@link GalleryEntry entries}
   * which can be navigated deeper into in a gallery
   * in a manner like a folder in a file-based operating system
   */
  variants?: GalleryEntries
}

/**
 * Defines the list of entries in a gallery,
 * a key-value object where the value is the entry,
 * and the key represents the id of a piece;
 * it is important for this to be uniquely defined at minimum within
 * the scope of the entry in relation to surrounding variants
 * in order for the app to properly navigate through variants
 * and ultimately the view page displaying the entry
 */
export type GalleryEntries = { [idOrTitle: string]: GalleryEntry }

/**
 * A wrapper around {@link GalleryEntries} for the app's use only which adds additional fields
 * in order for the app to effectively display the entries.
 */
export type GalleryDisplayedEntries = { [idOrTitle: string]: GalleryEntry & {
  /**
   * specifies whether the entry is hidden by the tags selected by a visitor
   */
  hidden?: boolean
}}

/**
 * Defines a tag used by entries in a gallery
 * used for filtering entries from view
 */
export type GalleryTag = {
  /**
   * specifies a category that the tag belongs to
   * in order to optionally organize them in the view;
   * if category is not specified, tags will be assigned
   * "NoCategory" and organized into a section without a heading
   * placed before any other sections formed by categories (if any)
   */
  category?: string
  /**
   * specifies the name that the tag will appear as in the DOM;
   * if not specified, the id of the tag will be used in its place
   */
  displayName?: string
}

/**
 * Defines the list of tags in a gallery,
 * a key-value object where the value is the entry,
 * and the key represents the id of a tag;
 * the id of a tag must be unique,
 * and the ids specified in a gallery entry must match
 * the ids specified in `GalleryTags` in order for them to work effectively
 */
export type GalleryTags = { [id: string]: GalleryTag }

/**
 * Defines the model of the `GalleryList` template
 */
export type GalleryList = {
  /**
   * the entries to display in a gallery-list template
   */
  entries: GalleryEntries
  /**
   * the tags to use for filtering entries
   */
  tags: GalleryTags
}
